//---------------------------------------------------------------------------


#pragma hdrstop

#include "Signal.h"
#include <stdlib.h>
#include <math.h>

//---------------------------------------------------------------------------

#pragma package(smart_init)

Signal::Signal()
{
	struct S_WAVE wave;
	wave.shape = WAVE_NONE;
	wave.fAmplitude = 1.0;
	wave.fFrequency = 1000.0;
	wave.fPhase = 0.0;
	for (int i=0; i<SHAPE_CNT; i++)
		shape.push_back(wave);
}

short* Signal::Generate(unsigned int count, float fDivX, float fDivY)
{
	if (count > sizeof(data)/sizeof(data[0]))
		return NULL;

	for (unsigned int i=0; i!=count; ++i)
	{
		fData[i] = 0;
	}

	for (unsigned int sig=0; sig<shape.size(); sig++)
	{
		struct S_WAVE wave = shape[sig];
		float period = 1.0/wave.fFrequency;
		float phase = wave.fPhase/360.0 * period;
		
		switch (wave.shape)
		{
		case WAVE_DC:
			for (unsigned int i=0; i!=count; ++i)
			{
				fData[i] += wave.fAmplitude/fDivY;
			}
			break;
		case WAVE_SINUS:
			for (unsigned int i=0; i!=count; ++i)
			{
				fData[i] += wave.fAmplitude/fDivY * sin(2.0 * M_PI * wave.fFrequency * (i * fDivX) + wave.fPhase/360.0);
			}
			break;
		case WAVE_SQUARE:
			for (unsigned int i=0; i!=count; ++i)
			{
				if ( ((int)(((i*fDivX) + phase)*2.0/period))%2 )
					fData[i] += wave.fAmplitude/fDivY;
				else
					fData[i] += (-1.0) * wave.fAmplitude/fDivY;
			}
			break;
		case WAVE_NOISE:
			for (unsigned int i=0; i!=count; ++i)
			{
				fData[i] += wave.fAmplitude/fDivY * (((float)random(100) - 50.0)/50.0);
			}
			break;
#if 0
		case WAVE_TRIANGLE:
			for (unsigned int i=0; i!=count; ++i)
			{
				if ( ((int)(((i*fDivX) + phase)*2.0/period))%2 )
				{
					// raising
					fData[i] += (-1.0) * wave.fAmplitude + ;
				}
				else
				{
                    // falling
					fData[i] += (-1.0) * wave.fAmplitude/fDivY;
				}
			}
			break;
#endif
		default:
			break;
		}
	}
             
	for (unsigned int i=0; i!=count; ++i)
	{
		data[i] = fData[i];
	}
	return data;
}


int Signal::GetTypeDescription(int type, std::string &desc)
{
	switch(type) {
	case	WAVE_NONE:
		desc = "none";
		break;
	case	WAVE_DC:
		desc = "DC offset";
		break;
	case	WAVE_SINUS:
		desc = "Sinus";
		break;
	case	WAVE_SQUARE:
		desc = "Square";
		break;
	case	WAVE_NOISE:
		desc = "Noise";
		break;
//	case	WAVE_TRIANGLE:
//		desc = "Triangle";
//		break;
	default:
		desc = "";
		return -1;
	}
	return 0;
}

